/************************************************************************
 *
 * Author : Jean-Francois Huppert & Pascal Vincent
 *          LPNHE-Paris
 * e-mail : huppert@lpnhep.in2p3.fr, vincentp@in2p3.fr 
 * Date   : 5 june 2001
 *
 ************************************************************************
 * FifoBlocks.c
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h> 
#include <sys/stat.h>
#include <fcntl.h> 
#include <unistd.h>
#include <errno.h>
#include <ctype.h>


#include "CtrlSecurityPilot.h"

void Tridiot() {
  volatile unsigned int 
    i,
    j,
    min,
    cur;

  int 
    raw[FIFO_BLOCK_FILE_SZ];

  for(i = 0; i < FIFO_BLOCK_FILE_SZ; i++) 
    { 
      if(Block_file[i].id == 0) raw[i] = -1; 
      else raw[i] = (int)Block_file[i].id;
      order[i] = -1;
    }

  for(i = 0; i < FIFO_BLOCK_FILE_SZ; i++) 
    {
      cur = 0;
      min = 0x7fffffff;
      for(j = 0; j < FIFO_BLOCK_FILE_SZ; j++) 
	{
	  if((raw[j] > 0) && (raw[j] < min)) 
	    {
	      min = raw[j];
	      cur = j;
	    }
	}
      if(min == 0x7fffffff) break;
      order[i] = cur;
      raw[cur] = -1;
  }
}


/************************************************************************/
/* open  Fifo_Block_file file */

int Get_Fifo_BLks(char *file_name) {

  if ((Blk_File = open(file_name, O_RDWR)) < 0) {

    /* file doesn't exist yet */
    if ((Blk_File = open(file_name, O_CREAT|O_RDWR, S_IRUSR | S_IWUSR)) < 0) {
      printf ("Can't create Fifo_Block_file : %s\n",file_name);
      return(5);
    }
    if (lseek(Blk_File, 0, SEEK_SET) < 0) {
      printf ("Can't initialize  Block_file (lseek failure): %s\n",file_name);
      close(Blk_File);  
      return(6);
    }
    if (write(Blk_File, Block_file, (size_t)sizeof(Block_file)) < 0) {
      printf ("Can't initialize  Block_file (write failure errno=%d): %s\n",
	      errno,file_name);
      perror("Get_Fifo_BLks");
      close(Blk_File);  
      return(6);
    }
   
  }

  else {

   /* file already in use */
    printf ("Block_file   used is : %s\n",file_name);

    /*
    if (lseek(Blk_File, 0, SEEK_SET) < 0) {
      printf ("Can't initialize Block_file (lseek failure): %s\n",file_name);
      close(Blk_File);  
      return(6);
    }
    */
    if (read(Blk_File, (void *)Block_file, (size_t)sizeof(Block_file)) < 0) {
      printf ("Can't read Block_file errno=%d : %s\n",errno,file_name);
      perror("Get_Fifo_BLks");
      close(Blk_File);  
      return(7);
    }
  }

  return 0;
}

/************************************************************************/
/* list  Fifo_Block_file file */

int List_Fifo_BLks(int nval) {

  int 
    j,
    i,
    wc,
    num,
    found,
    single;

  i = 0;
  num = 0;
  found = 0;
  single = 0;
  if((nval > 1) && (param > 0)) single = 1;
  
  if(Block_file[0].id == 0) {
    printf("\n no block defined in File : %s\n",Blk_File_name);
    return 0;
  }
  else 
    if(!single) printf("\n List of Fifo Blocks defined in File : %s\n",Blk_File_name);
  
  Tridiot();
	  
  while(i < FIFO_BLOCK_FILE_SZ) {
    if(order[i] < 0) break;
    else num = order[i];
	    
    if(!single) {
      if(Block_file[num].id != 0) {
	printf(" id %4x, lg %3d, %s",Block_file[num].id,Block_file[num].lg,Block_file[num].cmt);
	if(Block_file[num].cmt[strlen(Block_file[num].cmt) - 1] != (char) 0xa) printf("\n");
	found = 1;
      }
    }
    else {
      if(Block_file[num].id == param) {
	printf("\n %4x, lg %3d, %s",Block_file[num].id,Block_file[num].lg,Block_file[num].cmt);
	if(Block_file[num].cmt[strlen(Block_file[num].cmt) - 1] != (char) 0xa) printf("\n");
	wc = BLK_MAX_DATA;
	if(Block_file[num].lg < BLK_MAX_DATA) wc = Block_file[num].lg;
	for(j = 0; j < wc; j++) {
	  printf("  %16Lx",Block_file[num].data[j]);
	  if((j%4) == 3) printf("\n");
	}
	found = 1;
	break;
      }
    }
    i++;
  }

  if(!found) printf("\n block not found !\n");
  else printf("\n");

  if(!single) {return(-1);};
  
  return(num);
}

/************************************************************************
 * define  Fifo_Block 
 */

int Define_Fifo_BLk(int nval) {
	
  int 
    i,
    j,
    wc,
    nsscan,
    found;

  unsigned int
    d32;

  unsigned long long int
    d64;

  i = 0;
  found = 0;
  if((nval > 1) && (param > 0)) found = 1;

  while(i < FIFO_BLOCK_FILE_SZ) {
    if(!found) {
      if(Block_file[i].id == 0) {
	printf("\n define new block\n");
	printf(" (type one 'space' to abort)\n");
	d32 = 0;
	while(d32 == 0) {
	  printf("\n block_id (hexa) : ");
	  fgets(com,64,stdin);
	  wc = sscanf(com,"%x",&d32);
	  if(wc < 1) {
	    printf(" define block aborted !\n");
	    return 2;
	  }
	  if(d32 == 0) {
	    printf(" error : 0 is not a valid block_id !\n");
	    return 1;
	  }
	  for(j = 0; j < FIFO_BLOCK_FILE_SZ; j++) {
	    if(d32 == Block_file[j].id) {
	      printf(" error : block_id %x (hex) is already defined !\n",d32);
	      return 1;
	    }
	  }
	}
	printf(" summary : ");
	fgets(com,64,stdin);
	if(com[0] == ' ') {
	  printf(" define block aborted !\n");
	  return 2;
	}
	last_deleted = 0;
	last_ideleted = -1;
	Block_file[i].id = (unsigned short)d32;
	strncpy(Block_file[i].cmt,com,64);
	j=0;
	while(j < 64) if(com[j++] == '\0') break;
	while(j == 64) {
	  fgets(com,64,stdin);
	  j=0;
	  while(j < 64) if(com[j++] == '\0') break;
	}
	j = 0;
	printf(" enter data : %d 64-bits words maximum\n",BLK_MAX_DATA);
	while(j < BLK_MAX_DATA) {
	  printf(" (type '/' to quit)  data %d : ",j);
	  fgets(com,64,stdin);
	  nsscan = sscanf(com,"%Lx",&d64);
	  if(nsscan <= 0) break;
	  Block_file[i].data[j] = d64;
	  j++;
	}
	Block_file[i].lg = (unsigned short)j;
	if (lseek(Blk_File, 0, SEEK_SET) < 0) {
	  printf ("\nCan't rewind Block_file before updating (lseek failure %d): %s\n",
		  errno,Blk_File_name);
	  close(Blk_File);  
	  return(8);
	}
	else
	  if (write(Blk_File, (void *)Block_file, (size_t)sizeof(Block_file)) < 0) {
	    printf ("\nCan't update  Block_file (write failure): %s\n",Blk_File_name);
	    close(Blk_File);  
	    return(6);
	  }
	printf(" block %x (hex) is defined\n",Block_file[i].id);
	return 0;
      }
    }
    else {
      if(Block_file[i].id == param) {
	printf("\n modify block\n");
	printf(" id %4x lth %3d %s",Block_file[i].id,Block_file[i].lg,Block_file[i].cmt);
	printf("\n (type one 'space' to abort)\n");
	printf(" new summary : ");
	fgets(com,64,stdin);
	if(com[0] == ' ') {
	  printf(" redefine block aborted !\n");
	  return 2;
	}
	strncpy(Block_file[i].cmt,com,64);
	j=0;
	while(j < 64) if(com[j++] == '\0') break;
	while(j == 64) {
	  fgets(com,64,stdin);
	  j=0;
	  while(j < 64) if(com[j++] == '\0') break;
	}
	j = 0;
	printf(" new data (type '/' to quit)\n");
	while(j < BLK_MAX_DATA) {
	  printf(" (if OK => type '=') data %d  %Lx : ",j,Block_file[i].data[j]);
	  fgets(com,64,stdin);
	  nsscan = sscanf(com,"%Lx",&d64);
	  if(nsscan <= 0) {
	    if(com[0] != '=') break;
	    else d64 = Block_file[i].data[j];
	  }
	  Block_file[i].data[j] = d64;
	  j++;
	}
	Block_file[i].lg = (unsigned short)j;
	if (lseek(Blk_File, 0, SEEK_SET) < 0) {
	  printf ("\nCan't rewind Block_file before updating (lseek failure %d): %s\n",
		  errno,Blk_File_name);
	  close(Blk_File);  
	  return(8);
	}
	else
	  if (write(Blk_File, (void *)Block_file, (size_t)sizeof(Block_file)) < 0) {
	    printf ("\nCan't update  Block_file (write failure): %s\n",Blk_File_name);
	    close(Blk_File);  
	    return(6);
	  }
	printf("\n block %x (hex) is redefined\n",Block_file[i].id);
	return 0;
      }
    }
    i++;
  }
  if(found) printf(" block %x (hex) not found !\n",param);
  else printf(" File : %s  is full !\n",Blk_File_name);
  return 1;
}

/************************************************************************/
/* delete  Fifo_Block */

int Delete_Fifo_BLk(int nval) {

  int i;

  if(nval <= 1) {
    printf(" block_id is needed !\n\n");
    return 1;
  }

  if(param == 0) {                   /* undelete block case */
    if(last_deleted == 0) {
      printf(" nothing to undelete now !\n\n");
      return 2;
    }
    i = last_ideleted;
    if(Block_file[i].id != 0) {
      printf(" error : block_id 0x%x hasn't been deleted yet !\n\n",Block_file[i].id);
      return 2;
    }
    Block_file[i].id = last_deleted;
    if (lseek(Blk_File, 0, SEEK_SET) < 0) {
      printf ("\nCan't rewind Block_file before updating (lseek failure %d): %s\n",
	      errno,Blk_File_name);
      close(Blk_File);  
      return(8);
    }
    else
    if (write(Blk_File, (void *)Block_file, (size_t)sizeof(Block_file)) < 0) {
      printf ("\nCan't update  Block_file (write failure): %s\n",Blk_File_name);
      close(Blk_File);  
      return(6);
    }
    printf("\n Fifo_Block %x (hex) has been restored !\n",Block_file[i].id);
    printf(" id %4x, lg %3d, %s",Block_file[i].id,Block_file[i].lg,Block_file[i].cmt);
    last_deleted = 0;
    last_ideleted = -1;
    return 0;
  }
  else {                             /* delete block case */
    i = 0;
    while(i < FIFO_BLOCK_FILE_SZ) {
      if(Block_file[i].id == param) {
	printf("\n id %4x, lg %3d, %s",Block_file[i].id,Block_file[i].lg,Block_file[i].cmt);
	printf("\n delete ? ( y | n ) : ");
	fgets(com,64,stdin);
	if((tolower((int)com[0]) == 'y') ||(tolower((int)com[1]) == 'y') ) {
	  Block_file[i].id = 0;
	  last_deleted = param;
	  last_ideleted = i;
	  if (lseek(Blk_File, 0, SEEK_SET) < 0) {
	    printf ("\nCan't rewind Block_file before updating (lseek failure %d): %s\n",
		    errno,Blk_File_name);
	    close(Blk_File);  
	    return(8);
	  }
	  else
	  if (write(Blk_File, (void *)Block_file, (size_t)sizeof(Block_file)) < 0) {
	    printf ("\nCan't update  Block_file (write failure): %s\n",Blk_File_name);
	    close(Blk_File);  
	    return(6);
	  }
	  printf("\n Fifo_Block %x (hex) deleted !\n",param);
	}
	else
	  printf("\n Block has not been deleted !\n");
	return 0;
      }
      i++;
    }
    printf("\n block_id %x (hex) not found in file %s !\n",
	   param,Blk_File_name);
    return 1;
  };
};

/************************************************************************/
int Read_Scripts_List(int *nscripts) {
  int
    l,
    s;
  unsigned int 
    i,
    j,
    k;
  char
    *pt;

  if ((Scripts_List = open(FIFO_SCRIPT_LIST, O_RDONLY)) < 0) {
    /* file doesn't exist yet */
    printf ("%s not found : no scripts defined !\n",FIFO_SCRIPT_LIST);
    return(16);
  }

  else {
   /* file already in use */
    printf ("Scripts_list_File used is : %s\n",FIFO_SCRIPT_LIST);
    for(i = 0; i < SCRIPTS_MAX; i++) 
      for (j = 0; j < SCRIPT_NAME_LENGTH; j++) Script_name[i][j] = '\0';

    if (lseek(Scripts_List, 0, SEEK_SET) < 0) {
      printf ("Can't initialize Scripts_list (lseek failure): %s\n",FIFO_SCRIPT_LIST);
      return(16);
    }
    if ((l = read(Scripts_List, (void *)scrbuf, (size_t)sizeof(scrbuf))) <= 0) {
      printf ("Can't read Scripts_list_File  errno=%d : %s\n",errno,FIFO_SCRIPT_LIST);
      perror("Read_Scripts_List");
      return(17);
    }
  }
  scrbuf[l-1] = '\0';

  i = 0;
  j = 0;
  k = 0;
  while(scrbuf[i] != '\0') {
    if((scrbuf[i] != '\n') && (scrbuf[i] != '\a') && (scrbuf[i] != '\r')) {
      Script_name[k][j] = scrbuf[i];
      j++;
    }
    else {
      Script_name[k][j] = '\0';
      if((Script_name[k][0] != '#') && (Script_name[k][1] != '#')) {
	pt = strchr(Script_name[k],'.');
	if(pt != NULL) {
	  if(strcmp(".cust",pt) == 0) k++;
	}
      }
      j = 0;
    }
    i++;
  }
  *nscripts = k;

  for(i = 0; i <= k; i++) {
    strcpy(scrbuf,FIFO_SCRIPT_DIR);
    strcat(scrbuf,"/");
    strcat(scrbuf,Script_name[i]);
    if ((s = open(scrbuf, O_RDONLY)) < 0) {
      /* file doesn't exist yet */
      strcpy(scrcmt[i],"! file not found");
    }
    else {
      scrbuf[0] = '\0';
      l = read(s, (void *)scrbuf, 64);
      if (l <= 0) {
	strcpy(scrcmt[i],"! script_file is empty");
      }
      else {
	if((scrbuf[0] == '#') || (scrbuf[1] == '#')) {
	  for(j = 0; j < l; j++) {
	    if((scrbuf[j] == '\n') || (scrbuf[j] == '\a') || (scrbuf[j] == '\r')) {
	      scrbuf[j] = '\0';
	      break;
	    }
	  }
	  scrbuf[l-1] = '\0';
	  strcpy(scrcmt[i],scrbuf);
	}
	else strcpy(scrcmt[i],"! no summary found");
      }
      close(s);
    }
  }

  return 0;

/*
if ((wc = read(Script_File, (void *)scrbuf, (size_t)sizeof(scrbuf))) < 0) {
  printf ("Can't read Script_File  errno=%d : %s\n",errno,Script_name[param]);
}
scrbuf[wc-1] = '\0';
printf("\n script read\n%s\n",scrbuf);
if (lseek(Script_File, 0, SEEK_SET) < 0) {
  printf ("\nCan't rewind Script_file %s before executing (lseek failure %d)\n",
     Script_name[param],errno);
  break;
}
*/

}

/************************************************************************/











